import rox
from rox import choices, croak, g
import os, sys
import options

def choose_backdrop():
	# We've been called by the filer; print the backdrop to use.
	try:
		if options.mode == 'Random':
			choose_random()
		else:
			choose_advanced()
	except:
		rox.report_exception()

def choose_random():
	def choose():
		print choose_random_image()
	choose_on_timeout(choose)

def choose_on_timeout(callback):
	"""Call callback now and on each timeout (if set). callback may return a
	function which should be used to remove any temporary file."""
	from select import select
	time_between_updates = options.change_time.value * 60	# in s

	clean_fn = None
	try:
		while 1:
			clean_fn = callback()
			sys.stdout.flush()
			
			if sys.stdin.readline() != 'ok\n' or not time_between_updates:
				break

			if clean_fn: clean_fn()
			clean_fn = None

			ready = select([sys.stdin], [], [], time_between_updates)[0]
			if ready:
				break	# Ready to read end-of-file => pinboard closed
	finally:
		if clean_fn: clean_fn()

def choose_random_image():
	def images(path, style):
		if not path:
			return []
		ret = []
		for file in os.listdir(path):
			ret.append((os.path.join(path, file), style))
		return ret

	images = images(choices.load('Wallpaper', 'Centred'), 'centre') + \
		 images(choices.load('Wallpaper', 'Scaled'), 'scale') + \
		 images(choices.load('Wallpaper', 'Tiled'), 'tile')

	if not images:
		croak('No images found! Run Wallpaper and use Setup \
to choose some images.')

	import random

	image, style = random.choice(images)
	return "%s %s" % (style, image)

def create_tmp_file():
	import random
	t = 1
	while 1:
		tmpfile = '/tmp/wallpaper-' + `random.randint(1, 100000)`
		try:
			return (tmpfile,
				os.open(tmpfile, os.O_CREAT | os.O_EXCL | os.O_RDWR,
				        0600))
		except:
			if t < 10:
				t += 1
			else:
				raise

def choose_advanced():
	def choose():
		(tmpfile, out) = create_tmp_file()
		child = os.fork()
		if child == 0:
			try:
				if out != 1:
					os.dup2(out, 1)
					os.close(out)
				os.system(options.advanced_command)
				os._exit(0)
			finally:
				os._exit(1)
		os.close(out)
		(pid, status) = os.waitpid(child, 0)
		assert pid == child
		if os.WIFEXITED(status) and os.WEXITSTATUS(status):
			rox.alert("Command '%s' failed!" % options.advanced_command)
		print "centre", tmpfile
		def clean(): os.unlink(tmpfile)
		return clean
	choose_on_timeout(choose)



import rox
from rox import choices, g

mode = 'Random'
change_time = g.Adjustment(0, lower = 0, upper = 999, step_incr = 1)
advanced_command = 'xearth -png -nice 4'

try:
	options = choices.load('Wallpaper', 'Options')
	if options:
		for line in file(options):
			key, value = line.strip().split('=')
			if key == 'change_time':
				change_time.set_value(int(value))
			elif key == 'advanced_command':
				advanced_command = value
			elif key == 'mode':
				assert value in ('Random', 'Advanced')
				mode = value
except:
	rox.report_exception()

def save():
	path = choices.save('Wallpaper', 'Options')
	if not path:
		rox.croak("Can't save settings; saving is disabled by CHOICESPATH!")
	print >>file(path, 'w'), "mode=%s\nchange_time=%d\nadvanced_command=%s" % \
			(mode, change_time.value, advanced_command)



import os, sys

import rox
from rox import g, TRUE, FALSE, choices, filer
import actions
import options

tips = g.Tooltips()

def become_handler(mode):
	"Send a SOAP message to the filer so we become the default handler."
	options.mode = mode
	options.save()
	# Send message...
	app = os.path.abspath(os.path.dirname(sys.argv[0]))
	cin = os.popen('rox -R', 'w')
	cin.write("""<?xml version="1.0"?>
<env:Envelope xmlns:env="http://www.w3.org/2001/12/soap-envelope">
  <env:Body xmlns="http://rox.sourceforge.net/SOAP/ROX-Filer">
   <SetBackdropApp><App>""" + app + """</App></SetBackdropApp>
  </env:Body>
</env:Envelope>
""")

class Activate(g.Alignment):
	def __init__(self, mode):
		g.Alignment.__init__(self, 1, 1, 0, 0)
		button = rox.ButtonMixed(g.STOCK_APPLY, 'Activate %s mode' % mode.name)
		button.set_flags(g.CAN_DEFAULT)
		self.add(button)
		button.connect('clicked', lambda w: become_handler(mode.name))
		mode.pack_start(self, TRUE, TRUE, 0)
		button.grab_default()

class ChangeTime(g.HBox):
	def __init__(self):
		g.HBox.__init__(self, FALSE, 0)
		self.pack_start(g.Label('Change every'), FALSE, TRUE, 0)
		spin = g.SpinButton(options.change_time)
		self.pack_start(spin, FALSE, TRUE, 4)
		self.pack_start(g.Label('minutes'), FALSE, TRUE, 0)
		tips.set_tip(spin, 'The backdrop is changed each time the pinboard '
				   'is loaded (eg, when you log in). You can also get it '
				   'to change after a set interval by making this number '
				   'non-zero.')

class Advanced(g.VBox):
	name = 'Advanced'
	def __init__(self):
		g.VBox.__init__(self, FALSE, 4)
		self.set_border_width(4)

		label = g.Label('Enter a command to run which will write a backdrop image '
				'to its standard output stream. Eg: xearth -png -nice 4')
		label.set_selectable(TRUE)
		label.set_line_wrap(TRUE)
		self.pack_start(label, FALSE, TRUE, 0)

		hbox = g.HBox(FALSE, 0)
		hbox.pack_start(g.Label('Command:'), FALSE, TRUE, 0)

		command = g.Entry()
		command.set_text(options.advanced_command)
		command.grab_focus()
		def entry_changed(entry):
			options.advanced_command = entry.get_text()
		command.connect('changed', entry_changed)
		hbox.pack_start(command, TRUE, TRUE, 0)
		self.pack_start(hbox, FALSE, TRUE)

		self.pack_start(ChangeTime(), FALSE, TRUE, 0)

class RandomBackdrop(g.VBox):
	name = 'Random'

	def __init__(self):
		g.VBox.__init__(self, FALSE, 4)
		self.set_border_width(4)

		def endir(dir):
			if not os.path.exists(dir):
				os.mkdir(dir)
		def add_opendir(label, dir):
			button = g.Button(label)
			button_box.pack_start(button, FALSE, TRUE, 0)
			def open(button):
				path = choices.save('Wallpaper', dir)
				endir(path)
				filer.open_dir(path)
			button.connect('clicked', open)

		label = g.Label('Click on the buttons below to open your image '
				'directories. Copy, move or symlink images into '
				'each directory, depending on how you want each '
				'one displayed.')
		label.set_line_wrap(TRUE)
		self.pack_start(label, FALSE, TRUE, 0)

		align = g.Alignment(0.5, 0.5, 0, 0)
		self.pack_start(align, FALSE, TRUE, 0)
		button_box = g.VBox(TRUE, 2)
		align.add(button_box)
		add_opendir('Tiled images', 'Tiled')
		add_opendir('Scaled images', 'Scaled')
		add_opendir('Centred images', 'Centred')

		self.pack_start(ChangeTime(), FALSE, TRUE, 0)

modes = [RandomBackdrop(), Advanced()]

# From ROX-Lib, but let users off the upgrade tread-mill for one version ;-)
class Dialog(g.Dialog):
	"""This works in exactly the same way as a GtkDialog, except that
	it calls the toplevel_(un)ref functions for you automatically."""
	def __init__(*args, **kwargs):
		apply(g.Dialog.__init__, args, kwargs)
		rox.toplevel_ref()
		args[0].connect('destroy', rox.toplevel_unref)

class Window(Dialog):
	def __init__(self):
		Dialog.__init__(self)
		self.set_title('Wallpaper')
		self.set_position(g.WIN_POS_CENTER)
		self.set_has_separator(FALSE)

		# Add the buttons...

		self.add_button(g.STOCK_CLOSE, g.RESPONSE_CANCEL)

		def response(dialog, response):
			if response == g.RESPONSE_CANCEL:
				self.destroy()
		self.connect('response', response)

		notebook = g.Notebook()
		self.vbox.pack_start(notebook, TRUE, TRUE, 0)
		notebook.set_border_width(4)

		for m in modes:
			notebook.append_page(m, g.Label(m.name))
			Activate(m)

		self.vbox.show_all()

		try:
			i = [m.name for m in modes].index(options.mode)
		except:
			pass
		else:
			notebook.set_current_page(i)



# Most of the common code needed by ROX applications is in ROX-Lib2.
# Except this code, which is needed to find ROX-Lib2 in the first place!

# Just make sure you import findrox before importing anything inside
# ROX-Lib2...

import os, sys
from os.path import exists
import string

try:
	path = os.environ['LIBDIRPATH']
	paths = string.split(path, ':')
except KeyError:
	paths = [ os.environ['HOME'] + '/lib', '/usr/local/lib', '/usr/lib' ]

paths = map(lambda p: os.path.join(p, 'ROX-Lib2'), paths)
for p in paths:
	if exists(p):
		sys.path.append(os.path.join(p, 'python'))
		break
else:
	err = "This program needs ROX-Lib2 to run.\n" + \
		"I tried all of these places:\n\n" + \
	   	string.join(paths, '\n') + '\n\n' + \
		"ROX-Lib2 is available from:\nhttp://rox.sourceforge.net"
	try:
		sys.stderr.write('*** ' + err + '\n')
	except:
		pass
	try:
		import pygtk; pygtk.require('2.0')
		import gtk as g
	except:
		import gtk
		win = gtk.GtkDialog()
		message = gtk.GtkLabel(err + 
				'\n\nAlso, pygtk2 needs to be present')
		win.set_title('Missing ROX-Lib2')
		win.set_position(gtk.WIN_POS_CENTER)
		message.set_padding(20, 20)
		win.vbox.pack_start(message)

		ok = gtk.GtkButton("OK")
		ok.set_flags(gtk.CAN_DEFAULT)
		win.action_area.pack_start(ok)
		ok.connect('clicked', gtk.mainquit)
		ok.grab_default()
		
		win.connect('destroy', gtk.mainquit)
		win.show_all()
		gtk.mainloop()
	else:
		box = g.MessageDialog(None, g.MESSAGE_ERROR, 0,
					g.BUTTONS_OK, err)
		box.set_title('Missing ROX-Lib2')
		box.set_position(g.WIN_POS_CENTER)
		box.set_default_response(g.RESPONSE_OK)
		box.run()
	sys.exit(1)
